@@ -61,7 +61,7 @@ Here's the built-in JSON tool for an example::
     def json_in(force=True, debug=False):
         request = cherrypy.serving.request
         def json_processor(entity):
-            \"""Read application/json data into request.json.\"""
+            '''Read application/json data into request.json.'''
             if not entity.headers.get("Content-Length", ""):
                 raise cherrypy.HTTPError(411)
 
@@ -115,27 +115,29 @@ except ImportError:
 import re
 import sys
 import tempfile
-try:
-    from urllib import unquote_plus
-except ImportError:
-    def unquote_plus(bs):
-        """Bytes version of urllib.parse.unquote_plus."""
-        bs = bs.replace(ntob('+'), ntob(' '))
-        atoms = bs.split(ntob('%'))
-        for i in range(1, len(atoms)):
-            item = atoms[i]
-            try:
-                pct = int(item[:2], 16)
-                atoms[i] = bytes([pct]) + item[2:]
-            except ValueError:
-                pass
-        return ntob('').join(atoms)
+from urllib.parse import unquote
+
+import cheroot.server
 
 import cherrypy
-from cherrypy._cpcompat import text_or_bytes, ntob, ntou
+from cherrypy._cpcompat import ntou
 from cherrypy.lib import httputil
 
 
+def unquote_plus(bs):
+    """Bytes version of urllib.parse.unquote_plus."""
+    bs = bs.replace(b'+', b' ')
+    atoms = bs.split(b'%')
+    for i in range(1, len(atoms)):
+        item = atoms[i]
+        try:
+            pct = int(item[:2], 16)
+            atoms[i] = bytes([pct]) + item[2:]
+        except ValueError:
+            pass
+    return b''.join(atoms)
+
+
 # ------------------------------- Processors -------------------------------- #
 
 def process_urlencoded(entity):
@@ -144,14 +146,14 @@ def process_urlencoded(entity):
     for charset in entity.attempt_charsets:
         try:
             params = {}
-            for aparam in qs.split(ntob('&')):
-                for pair in aparam.split(ntob(';')):
+            for aparam in qs.split(b'&'):
+                for pair in aparam.split(b';'):
                     if not pair:
                         continue
 
-                    atoms = pair.split(ntob('='), 1)
+                    atoms = pair.split(b'=', 1)
                     if len(atoms) == 1:
-                        atoms.append(ntob(''))
+                        atoms.append(b'')
 
                     key = unquote_plus(atoms[0]).decode(charset)
                     value = unquote_plus(atoms[1]).decode(charset)
@@ -315,7 +317,8 @@ class Entity(object):
     :attr:`request.body.parts<cherrypy._cpreqbody.Entity.parts>`. You can
     enable it with::
 
-        cherrypy.request.body.processors['multipart'] = _cpreqbody.process_multipart
+        cherrypy.request.body.processors['multipart'] = \
+            _cpreqbody.process_multipart
 
     in an ``on_start_resource`` tool.
     """
@@ -325,14 +328,15 @@ class Entity(object):
     # absence of a charset parameter, is US-ASCII."
     # However, many browsers send data in utf-8 with no charset.
     attempt_charsets = ['utf-8']
-    """A list of strings, each of which should be a known encoding.
+    r"""A list of strings, each of which should be a known encoding.
 
     When the Content-Type of the request body warrants it, each of the given
     encodings will be tried in order. The first one to successfully decode the
     entity without raising an error is stored as
     :attr:`entity.charset<cherrypy._cpreqbody.Entity.charset>`. This defaults
     to ``['utf-8']`` (plus 'ISO-8859-1' for "text/\*" types, as required by
-    `HTTP/1.1 <http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.7.1>`_),
+    `HTTP/1.1
+    <http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.7.1>`_),
     but ``['us-ascii', 'utf-8']`` for multipart parts.
     """
 
@@ -465,13 +469,10 @@ class Entity(object):
                     self.filename.endswith('"')
                 ):
                     self.filename = self.filename[1:-1]
-
-    # The 'type' attribute is deprecated in 3.2; remove it in 3.3.
-    type = property(
-        lambda self: self.content_type,
-        doc='A deprecated alias for '
-            ':attr:`content_type<cherrypy._cpreqbody.Entity.content_type>`.'
-    )
+            if 'filename*' in disp.params:
+                # @see https://tools.ietf.org/html/rfc5987
+                encoding, lang, filename = disp.params['filename*'].split("'")
+                self.filename = unquote(str(filename), encoding)
 
     def read(self, size=None, fp_out=None):
         return self.fp.read(size, fp_out)
@@ -523,7 +524,7 @@ class Entity(object):
         value = self.decode_entity(value)
         return value
 
-    def decode_entity(self , value):
+    def decode_entity(self, value):
         """Return a given byte encoded value as a string"""
         for charset in self.attempt_charsets:
             try:
@@ -574,14 +575,15 @@ class Part(Entity):
     # "The default character set, which must be assumed in the absence of a
     # charset parameter, is US-ASCII."
     attempt_charsets = ['us-ascii', 'utf-8']
-    """A list of strings, each of which should be a known encoding.
+    r"""A list of strings, each of which should be a known encoding.
 
     When the Content-Type of the request body warrants it, each of the given
     encodings will be tried in order. The first one to successfully decode the
     entity without raising an error is stored as
     :attr:`entity.charset<cherrypy._cpreqbody.Entity.charset>`. This defaults
     to ``['utf-8']`` (plus 'ISO-8859-1' for "text/\*" types, as required by
-    `HTTP/1.1 <http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.7.1>`_),
+    `HTTP/1.1
+    <http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.7.1>`_),
     but ``['us-ascii', 'utf-8']`` for multipart parts.
     """
 
@@ -627,17 +629,17 @@ class Part(Entity):
                 # No more data--illegal end of headers
                 raise EOFError('Illegal end of headers.')
 
-            if line == ntob('\r\n'):
+            if line == b'\r\n':
                 # Normal end of headers
                 break
-            if not line.endswith(ntob('\r\n')):
+            if not line.endswith(b'\r\n'):
                 raise ValueError('MIME requires CRLF terminators: %r' % line)
 
-            if line[0] in ntob(' \t'):
+            if line[0] in b' \t':
                 # It's a continuation line.
                 v = line.strip().decode('ISO-8859-1')
             else:
-                k, v = line.split(ntob(':'), 1)
+                k, v = line.split(b':', 1)
                 k = k.strip().decode('ISO-8859-1')
                 v = v.strip().decode('ISO-8859-1')
 
@@ -658,8 +660,8 @@ class Part(Entity):
         object that supports the 'write' method; all bytes read will be
         written to the fp, and that fp is returned.
         """
-        endmarker = self.boundary + ntob('--')
-        delim = ntob('')
+        endmarker = self.boundary + b'--'
+        delim = b''
         prev_lf = True
         lines = []
         seen = 0
@@ -667,7 +669,7 @@ class Part(Entity):
             line = self.fp.readline(1 << 16)
             if not line:
                 raise EOFError('Illegal end of multipart body.')
-            if line.startswith(ntob('--')) and prev_lf:
+            if line.startswith(b'--') and prev_lf:
                 strippedline = line.strip()
                 if strippedline == self.boundary:
                     break
@@ -677,16 +679,16 @@ class Part(Entity):
 
             line = delim + line
 
-            if line.endswith(ntob('\r\n')):
-                delim = ntob('\r\n')
+            if line.endswith(b'\r\n'):
+                delim = b'\r\n'
                 line = line[:-2]
                 prev_lf = True
-            elif line.endswith(ntob('\n')):
-                delim = ntob('\n')
+            elif line.endswith(b'\n'):
+                delim = b'\n'
                 line = line[:-1]
                 prev_lf = True
             else:
-                delim = ntob('')
+                delim = b''
                 prev_lf = False
 
             if fp_out is None:
@@ -700,7 +702,7 @@ class Part(Entity):
                 fp_out.write(line)
 
         if fp_out is None:
-            result = ntob('').join(lines)
+            result = b''.join(lines)
             return result
         else:
             fp_out.seek(0)
@@ -715,7 +717,7 @@ class Part(Entity):
             self.file = self.read_into_file()
         else:
             result = self.read_lines_to_boundary()
-            if isinstance(result, text_or_bytes):
+            if isinstance(result, bytes):
                 self.value = result
             else:
                 self.file = result
@@ -730,20 +732,11 @@ class Part(Entity):
         self.read_lines_to_boundary(fp_out=fp_out)
         return fp_out
 
+
 Entity.part_class = Part
 
 inf = float('inf')
 
-comma_separated_headers = [
-    'Accept', 'Accept-Charset', 'Accept-Encoding',
-    'Accept-Language', 'Accept-Ranges', 'Allow',
-    'Cache-Control', 'Connection', 'Content-Encoding',
-    'Content-Language', 'Expect', 'If-Match',
-    'If-None-Match', 'Pragma', 'Proxy-Authenticate',
-    'Te', 'Trailer', 'Transfer-Encoding', 'Upgrade',
-    'Vary', 'Via', 'Warning', 'Www-Authenticate'
-]
-
 
 class SizedReader:
 
@@ -753,7 +746,7 @@ class SizedReader:
         self.fp = fp
         self.length = length
         self.maxbytes = maxbytes
-        self.buffer = ntob('')
+        self.buffer = b''
         self.bufsize = bufsize
         self.bytes_read = 0
         self.done = False
@@ -789,7 +782,7 @@ class SizedReader:
         if remaining == 0:
             self.finish()
             if fp_out is None:
-                return ntob('')
+                return b''
             else:
                 return None
 
@@ -799,7 +792,7 @@ class SizedReader:
         if self.buffer:
             if remaining is inf:
                 data = self.buffer
-                self.buffer = ntob('')
+                self.buffer = b''
             else:
                 data = self.buffer[:remaining]
                 self.buffer = self.buffer[remaining:]
@@ -848,7 +841,7 @@ class SizedReader:
                 fp_out.write(data)
 
         if fp_out is None:
-            return ntob('').join(chunks)
+            return b''.join(chunks)
 
     def readline(self, size=None):
         """Read a line from the request body and return it."""
@@ -860,7 +853,7 @@ class SizedReader:
             data = self.read(chunksize)
             if not data:
                 break
-            pos = data.find(ntob('\n')) + 1
+            pos = data.find(b'\n') + 1
             if pos:
                 chunks.append(data[:pos])
                 remainder = data[pos:]
@@ -869,7 +862,7 @@ class SizedReader:
                 break
             else:
                 chunks.append(data)
-        return ntob('').join(chunks)
+        return b''.join(chunks)
 
     def readlines(self, sizehint=None):
         """Read lines from the request body and return them."""
@@ -898,21 +891,21 @@ class SizedReader:
 
             try:
                 for line in self.fp.read_trailer_lines():
-                    if line[0] in ntob(' \t'):
+                    if line[0] in b' \t':
                         # It's a continuation line.
                         v = line.strip()
                     else:
                         try:
-                            k, v = line.split(ntob(':'), 1)
+                            k, v = line.split(b':', 1)
                         except ValueError:
                             raise ValueError('Illegal header line.')
                         k = k.strip().title()
                         v = v.strip()
 
-                    if k in comma_separated_headers:
-                        existing = self.trailers.get(envname)
+                    if k in cheroot.server.comma_separated_headers:
+                        existing = self.trailers.get(k)
                         if existing:
-                            v = ntob(', ').join((existing, v))
+                            v = b', '.join((existing, v))
                     self.trailers[k] = v
             except Exception:
                 e = sys.exc_info()[1]
@@ -992,12 +985,6 @@ class RequestBody(Entity):
         # add them in here.
         request_params = self.request_params
         for key, value in self.params.items():
-            # Python 2 only: keyword arguments must be byte strings (type
-            # 'str').
-            if sys.version_info < (3, 0):
-                if isinstance(key, unicode):
-                    key = key.encode('ISO-8859-1')
-
             if key in request_params:
                 if not isinstance(request_params[key], list):
                     request_params[key] = [request_params[key]]
